'''
Repository for simple color data such as discrete color lists
and custom colormaps

Created on Dec 1, 2014

@author: gdelraye
'''
import base_objects
import collections

#--- Custom colormaps:
'''
The intention for these colormaps is that the ordering of the colors should be
intuitive, the lightest color should be distinguishable from a white background,
and the gradients in lightness and hue should be smooth.


Table of colormap descriptions:

name            color progression             intended use
-----------------------------------------------------------------------------
low2high        blue - red                    general
cool2hot        black - red - gold            temperature or heat
deep2shallow    black - teal - gold           bathymetry maps
dirty2clean     olive - grey - sky blue       environmental condition or degredation
sparse2dense    blue - green - yellow - red   density maps

Access Matplotlib colormap objects as e.g. custom_cmaps.low2high
'''
endcolors = [((0.0, 0.40533333333333332, 0.80000000000000004),
              (0.63396840061025794, 0.53760761109177391, 0.83219631047685338),
              (0.97843137254901957, 0.1488917306052856, 0.0)),
             ((0.0, 0.0, 0.0),
              (1.0, 0.0, 0.0),
              (1.0, 0.84313725490196079, 0.0)),
             ((0.0, 0.0, 0.0),
              (0.0, 0.5019607843137255, 0.5019607843137255),
              (1.0, 0.8431372549019608, 0.0)),
             ((0.20196078431372544, 0.20196078431372549, 0.0),
              (0.5117647058823529, 0.5117647058823529, 0.5117647058823529),
              (0.19999999999999996, 0.79921568627450978, 1.0)),
             ((0.0, 0.35466666666666657, 0.69999999999999996),
              (0.19529411764705884, 0.69803921568627447, 0.67049153908138581),
              (0.84999999999999998, 0.71666666666666656, 0.0),
              (0.97843137254901957, 0.1488917306052856, 0.0))]
names = ('low2high', 'cool2hot', 'deep2shallow', 'dirty2clean', 'sparse2dense')
cmaps = [base_objects.rgba2colorlist(colors).LinearSegmentedColormap() for colors in endcolors]
cmap_dict = dict(zip(names, cmaps))
custom_colormaps = collections.namedtuple('colormaps', cmap_dict)(**cmap_dict)

#--- Sets of maximally contrasting colors
'''
The intention of these color lists are for each color to be maximally distinct from every other
color in the set.

Table of color list descriptions:

name                number of colors     origin
-------------------------------------------------------------------------------------------------
cartercarter_6      6                    Carter, R. C. and Carter, E. C. 1982. High-Contrast Sets 
                                           of Colours. Applied Optics 21(16): 2936-2939.
greenarmytage_26    26                   Green-Armytage, P. 2010. A Colour Alphabet and the Limits
                                           of Colour Coding. Color: Design and Creativity 5(10):
                                           1-23.
tatarize_269        269                  Downloaded from <http://godsnotwheregodsnot.blogspot.ru/20
                                           13/11/kmeans-color-quantization-seeding.html> on December
                                           1st, 2014.
                                           
Access list of rgb tuples as e.g. custom_colorsets.cartercarter_6
'''

colors = [((0.9333333333333333, 0.2196078431372549, 0.8274509803921568),
           (0.0, 0.27450980392156865, 0.9529411764705882),
           (1.0, 0.7333333333333333, 0.2980392156862745),
           (0.1843137254901961, 0.984313725490196, 0.19215686274509805),
           (1.0, 0.0, 0.2627450980392157),
           (0.6862745098039216, 0.9450980392156862, 0.9764705882352941)),
          ((0.9411764705882353, 0.6392156862745098, 1.0), 
           (0.0, 0.4588235294117647, 0.8627450980392157), 
           (0.6, 0.24705882352941178, 0.0), 
           (0.2980392156862745, 0.0, 0.3607843137254902), 
           (0.09803921568627451, 0.09803921568627451, 0.09803921568627451), 
           (0.0, 0.3607843137254902, 0.19215686274509805), 
           (0.16862745098039217, 0.807843137254902, 0.2823529411764706), 
           (1.0, 0.8, 0.6), 
           (0.5019607843137255, 0.5019607843137255, 0.5019607843137255), 
           (0.5803921568627451, 1.0, 0.7098039215686275), 
           (0.5607843137254902, 0.48627450980392156, 0.0), 
           (0.615686274509804, 0.8, 0.0), 
           (0.7607843137254902, 0.0, 0.5333333333333333), 
           (0.0, 0.2, 0.5019607843137255), 
           (1.0, 0.6431372549019608, 0.0196078431372549), 
           (1.0, 0.6588235294117647, 0.7333333333333333), 
           (0.25882352941176473, 0.4, 0.0), 
           (1.0, 0.0, 0.06274509803921569), 
           (0.3686274509803922, 0.9450980392156862, 0.9490196078431372), 
           (0.0, 0.6, 0.5607843137254902), 
           (0.8784313725490196, 1.0, 0.4), 
           (0.4549019607843137, 0.0392156862745098, 1.0), 
           (0.6, 0.0, 0.0), 
           (1.0, 1.0, 0.5019607843137255), 
           (1.0, 1.0, 0.0), 
           (1.0, 0.3137254901960784, 0.0196078431372549)),
          ((0.0, 0.0, 0.0),
           (1.0, 1.0, 0.0),
           (0.10980392156862745, 0.9019607843137255, 1.0),
            (1.0, 0.20392156862745098, 1.0),
            (1.0, 0.2901960784313726, 0.27450980392156865),
            (0.0, 0.5372549019607843, 0.2549019607843137),
            (0.0, 0.43529411764705883, 0.6509803921568628),
            (0.6392156862745098, 0.0, 0.34901960784313724),
            (1.0, 0.8588235294117647, 0.8980392156862745),
            (0.47843137254901963, 0.28627450980392155, 0.0),
            (0.0, 0.0, 0.6509803921568628),
            (0.38823529411764707, 1.0, 0.6745098039215687),
            (0.7176470588235294, 0.592156862745098, 0.3843137254901961),
            (0.0, 0.30196078431372547, 0.2627450980392157),
            (0.5607843137254902, 0.6901960784313725, 1.0),
            (0.6, 0.49019607843137253, 0.5294117647058824),
            (0.35294117647058826, 0.0, 0.027450980392156862),
            (0.5019607843137255, 0.5882352941176471, 0.5764705882352941),
            (0.996078431372549, 1.0, 0.9019607843137255),
            (0.10588235294117647, 0.26666666666666666, 0.0),
            (0.30980392156862746, 0.7764705882352941, 0.00392156862745098),
            (0.23137254901960785, 0.36470588235294116, 1.0),
            (0.2901960784313726, 0.23137254901960785, 0.3254901960784314),
            (1.0, 0.1843137254901961, 0.5019607843137255),
            (0.3803921568627451, 0.3803921568627451, 0.35294117647058826),
            (0.7294117647058823, 0.03529411764705882, 0.0),
            (0.4196078431372549, 0.4745098039215686, 0.0),
            (0.0, 0.7607843137254902, 0.6274509803921569),
            (1.0, 0.6666666666666666, 0.5725490196078431),
            (1.0, 0.5647058823529412, 0.788235294117647),
            (0.7254901960784313, 0.011764705882352941, 0.6666666666666666),
            (0.8196078431372549, 0.3803921568627451, 0.0),
            (0.8666666666666667, 0.9372549019607843, 1.0),
            (0.0, 0.0, 0.20784313725490197),
            (0.4823529411764706, 0.30980392156862746, 0.29411764705882354),
            (0.6313725490196078, 0.7607843137254902, 0.6),
            (0.18823529411764706, 0.0, 0.09411764705882353),
            (0.0392156862745098, 0.6509803921568628, 0.8470588235294118),
            (0.00392156862745098, 0.2, 0.28627450980392155),
            (0.0, 0.5176470588235295, 0.43529411764705883),
            (0.21568627450980393, 0.12941176470588237, 0.00392156862745098),
            (1.0, 0.7098039215686275, 0.0),
            (0.7607843137254902, 1.0, 0.9294117647058824),
            (0.6274509803921569, 0.4745098039215686, 0.7490196078431373),
            (0.8, 0.027450980392156862, 0.26666666666666666),
            (0.7529411764705882, 0.7254901960784313, 0.6980392156862745),
            (0.7607843137254902, 1.0, 0.6),
            (0.0, 0.11764705882352941, 0.03529411764705882),
            (0.0, 0.2823529411764706, 0.611764705882353),
            (0.43529411764705883, 0.0, 0.3843137254901961),
            (0.047058823529411764, 0.7411764705882353, 0.4),
            (0.9333333333333333, 0.7647058823529411, 1.0),
            (0.27058823529411763, 0.42745098039215684, 0.4588235294117647),
            (0.7176470588235294, 0.4823529411764706, 0.40784313725490196),
            (0.47843137254901963, 0.5294117647058824, 0.6313725490196078),
            (0.47058823529411764, 0.5529411764705883, 0.4),
            (0.5333333333333333, 0.3333333333333333, 0.47058823529411764),
            (0.9803921568627451, 0.8156862745098039, 0.6235294117647059),
            (1.0, 0.5411764705882353, 0.6039215686274509),
            (0.8196078431372549, 0.3411764705882353, 0.6274509803921569),
            (0.7450980392156863, 0.7686274509803922, 0.34901960784313724),
            (0.27058823529411763, 0.4, 0.2823529411764706),
            (0.0, 0.5254901960784314, 0.9294117647058824),
            (0.5333333333333333, 0.43529411764705883, 0.2980392156862745),
            (0.20392156862745098, 0.21176470588235294, 0.17647058823529413),
            (0.7058823529411765, 0.6588235294117647, 0.7411764705882353),
            (0.0, 0.6509803921568628, 0.6666666666666666),
            (0.27058823529411763, 0.17254901960784313, 0.17254901960784313),
            (0.38823529411764707, 0.38823529411764707, 0.4588235294117647),
            (0.6392156862745098, 0.7843137254901961, 0.788235294117647),
            (1.0, 0.5686274509803921, 0.24705882352941178),
            (0.5764705882352941, 0.5411764705882353, 0.5058823529411764),
            (0.3411764705882353, 0.3254901960784314, 0.1607843137254902),
            (0.0, 0.996078431372549, 0.8117647058823529),
            (0.6901960784313725, 0.3568627450980392, 0.43529411764705883),
            (0.5490196078431373, 0.8156862745098039, 1.0),
            (0.23137254901960785, 0.592156862745098, 0.0),
            (0.01568627450980392, 0.9686274509803922, 0.3411764705882353),
            (0.7843137254901961, 0.6313725490196078, 0.6313725490196078),
            (0.11764705882352941, 0.43137254901960786, 0.0),
            (0.4745098039215686, 0.0, 0.8431372549019608),
            (0.6549019607843137, 0.4588235294117647, 0.0),
            (0.38823529411764707, 0.403921568627451, 0.6627450980392157),
            (0.6274509803921569, 0.34509803921568627, 0.21568627450980393),
            (0.4196078431372549, 0.0, 0.17254901960784313),
            (0.4666666666666667, 0.14901960784313725, 0.0),
            (0.8431372549019608, 0.5647058823529412, 1.0),
            (0.6078431372549019, 0.592156862745098, 0.0),
            (0.32941176470588235, 0.6196078431372549, 0.4745098039215686),
            (1.0, 0.9647058823529412, 0.6235294117647059),
            (0.12549019607843137, 0.08627450980392157, 0.1450980392156863),
            (0.4470588235294118, 0.2549019607843137, 0.5607843137254902),
            (0.7372549019607844, 0.13725490196078433, 1.0),
            (0.6, 0.6784313725490196, 0.7529411764705882),
            (0.22745098039215686, 0.1411764705882353, 0.396078431372549),
            (0.5725490196078431, 0.13725490196078433, 0.1607843137254902),
            (0.3568627450980392, 0.27058823529411763, 0.20392156862745098),
            (0.9921568627450981, 0.9098039215686274, 0.8627450980392157),
            (0.25098039215686274, 0.3058823529411765, 0.3333333333333333),
            (0.0, 0.5372549019607843, 0.6392156862745098),
            (0.796078431372549, 0.49411764705882355, 0.596078431372549),
            (0.6431372549019608, 0.9098039215686274, 0.01568627450980392),
            (0.19607843137254902, 0.3058823529411765, 0.4470588235294118),
            (0.41568627450980394, 0.22745098039215686, 0.2980392156862745),
            (0.5137254901960784, 0.6705882352941176, 0.34509803921568627),
            (0.0, 0.10980392156862745, 0.11764705882352941),
            (0.8196078431372549, 0.9686274509803922, 0.807843137254902),
            (0.0, 0.29411764705882354, 0.1568627450980392),
            (0.7843137254901961, 0.8156862745098039, 0.9647058823529412),
            (0.6392156862745098, 0.6431372549019608, 0.5372549019607843),
            (0.5019607843137255, 0.4235294117647059, 0.4),
            (0.13333333333333333, 0.1568627450980392, 0.0),
            (0.7490196078431373, 0.33725490196078434, 0.3137254901960784),
            (0.9098039215686274, 0.18823529411764706, 0.0),
            (0.4, 0.4745098039215686, 0.42745098039215684),
            (0.8549019607843137, 0.0, 0.48627450980392156),
            (1.0, 0.10196078431372549, 0.34901960784313724),
            (0.5411764705882353, 0.8588235294117647, 0.7058823529411765),
            (0.11764705882352941, 0.00784313725490196, 0.0),
            (0.3568627450980392, 0.3058823529411765, 0.3176470588235294),
            (0.7843137254901961, 0.5843137254901961, 0.7725490196078432),
            (0.19607843137254902, 0.0, 0.2),
            (1.0, 0.40784313725490196, 0.19607843137254902),
            (0.4, 0.8823529411764706, 0.8274509803921568),
            (0.8117647058823529, 0.803921568627451, 0.6745098039215687),
            (0.8156862745098039, 0.6745098039215687, 0.5803921568627451),
            (0.49411764705882355, 0.8274509803921568, 0.4745098039215686),
            (0.00392156862745098, 0.17254901960784313, 0.34509803921568627),
            (0.47843137254901963, 0.4823529411764706, 1.0),
            (0.8392156862745098, 0.5568627450980392, 0.00392156862745098),
            (0.20784313725490197, 0.2, 0.2235294117647059),
            (0.47058823529411764, 0.6862745098039216, 0.6313725490196078),
            (0.996078431372549, 0.6980392156862745, 0.7764705882352941),
            (0.4588235294117647, 0.4745098039215686, 0.48627450980392156),
            (0.5137254901960784, 0.45098039215686275, 0.5764705882352941),
            (0.5803921568627451, 0.22745098039215686, 0.30196078431372547),
            (0.7098039215686275, 0.9568627450980393, 1.0),
            (0.8235294117647058, 0.8627450980392157, 0.8352941176470589),
            (0.5843137254901961, 0.33725490196078434, 0.7411764705882353),
            (0.41568627450980394, 0.44313725490196076, 0.2901960784313726),
            (0.0, 0.07450980392156863, 0.1450980392156863),
            (0.00784313725490196, 0.3215686274509804, 0.37254901960784315),
            (0.0392156862745098, 0.6392156862745098, 0.9686274509803922),
            (0.9137254901960784, 0.5058823529411764, 0.4627450980392157),
            (0.8588235294117647, 0.8352941176470589, 0.8666666666666667),
            (0.3686274509803922, 0.7372549019607844, 0.8196078431372549),
            (0.23921568627450981, 0.30980392156862746, 0.26666666666666666),
            (0.49411764705882355, 0.39215686274509803, 0.0196078431372549),
            (0.00784313725490196, 0.40784313725490196, 0.3058823529411765),
            (0.5882352941176471, 0.16862745098039217, 0.4588235294117647),
            (0.5529411764705883, 0.5215686274509804, 0.27450980392156865),
            (0.5882352941176471, 0.5843137254901961, 0.7725490196078432),
            (0.9058823529411765, 0.45098039215686275, 0.807843137254902),
            (0.8470588235294118, 0.41568627450980394, 0.47058823529411764),
            (0.24313725490196078, 0.5372549019607843, 0.7450980392156863),
            (0.792156862745098, 0.5137254901960784, 0.3058823529411765),
            (0.3176470588235294, 0.5411764705882353, 0.5294117647058824),
            (0.3568627450980392, 0.06666666666666667, 0.23529411764705882),
            (0.3333333333333333, 0.5058823529411764, 0.23137254901960785),
            (0.9058823529411765, 0.01568627450980392, 0.7686274509803922),
            (0.0, 0.0, 0.37254901960784315),
            (0.6627450980392157, 0.45098039215686275, 0.6),
            (0.29411764705882354, 0.5058823529411764, 0.3764705882352941),
            (0.34901960784313724, 0.45098039215686275, 0.5411764705882353),
            (1.0, 0.36470588235294116, 0.6549019607843137),
            (0.9686274509803922, 0.788235294117647, 0.7490196078431373),
            (0.39215686274509803, 0.19215686274509805, 0.15294117647058825),
            (0.3176470588235294, 0.22745098039215686, 0.00392156862745098),
            (0.4196078431372549, 0.5803921568627451, 0.6666666666666666),
            (0.3176470588235294, 0.6274509803921569, 0.34509803921568627),
            (0.6431372549019608, 0.3568627450980392, 0.00784313725490196),
            (0.11372549019607843, 0.09019607843137255, 0.00784313725490196),
            (0.8862745098039215, 0.0, 0.15294117647058825),
            (0.9058823529411765, 0.6705882352941176, 0.38823529411764707),
            (0.2980392156862745, 0.3764705882352941, 0.00392156862745098),
            (0.611764705882353, 0.4117647058823529, 0.4),
            (0.39215686274509803, 0.32941176470588235, 0.4823529411764706),
            (0.592156862745098, 0.592156862745098, 0.6196078431372549),
            (0.0, 0.41568627450980394, 0.4),
            (0.2235294117647059, 0.0784313725490196, 0.023529411764705882),
            (0.9568627450980393, 0.8431372549019608, 0.28627450980392155),
            (0.0, 0.27058823529411763, 0.8235294117647058),
            (0.0, 0.4235294117647059, 0.19215686274509805),
            (0.8666666666666667, 0.7137254901960784, 0.8156862745098039),
            (0.48627450980392156, 0.396078431372549, 0.44313725490196076),
            (0.6235294117647059, 0.6980392156862745, 0.6431372549019608),
            (0.0, 0.8470588235294118, 0.5686274509803921),
            (0.08235294117647059, 0.6274509803921569, 0.5411764705882353),
            (0.7372549019607844, 0.396078431372549, 0.9137254901960784),
            (1.0, 1.0, 0.996078431372549),
            (0.7764705882352941, 0.8627450980392157, 0.6),
            (0.12549019607843137, 0.23137254901960785, 0.23529411764705882),
            (0.403921568627451, 0.06666666666666667, 0.5647058823529412),
            (0.4196078431372549, 0.22745098039215686, 0.39215686274509803),
            (0.9607843137254902, 0.8823529411764706, 1.0),
            (1.0, 0.6274509803921569, 0.9490196078431372),
            (0.8, 0.6666666666666666, 0.20784313725490197),
            (0.21568627450980393, 0.27058823529411763, 0.15294117647058825),
            (0.5450980392156862, 0.7058823529411765, 0.0),
            (0.4745098039215686, 0.47058823529411764, 0.40784313725490196),
            (0.7764705882352941, 0.0, 0.35294117647058826),
            (0.23137254901960785, 0.0, 0.0392156862745098),
            (0.7843137254901961, 0.3843137254901961, 0.25098039215686274),
            (0.1607843137254902, 0.3764705882352941, 0.48627450980392156),
            (0.25098039215686274, 0.13725490196078433, 0.20392156862745098),
            (0.49019607843137253, 0.35294117647058826, 0.26666666666666666),
            (0.8, 0.7215686274509804, 0.48627450980392156),
            (0.7215686274509804, 0.5058823529411764, 0.5137254901960784),
            (0.6666666666666666, 0.3176470588235294, 0.6),
            (0.7098039215686275, 0.8392156862745098, 0.7647058823529411),
            (0.6392156862745098, 0.5176470588235295, 0.4117647058823529),
            (0.6235294117647059, 0.5803921568627451, 0.9411764705882353),
            (0.6549019607843137, 0.27058823529411763, 0.44313725490196076),
            (0.7215686274509804, 0.5803921568627451, 0.6509803921568628),
            (0.44313725490196076, 0.7333333333333333, 0.5490196078431373),
            (0.0, 0.7058823529411765, 0.2),
            (0.47058823529411764, 0.6196078431372549, 0.788235294117647),
            (0.42745098039215684, 0.5019607843137255, 0.7294117647058823),
            (0.5843137254901961, 0.24705882352941178, 0.0),
            (0.3686274509803922, 1.0, 0.011764705882352941),
            (0.8941176470588236, 1.0, 0.9882352941176471),
            (0.10588235294117647, 0.8823529411764706, 0.4666666666666667),
            (0.7372549019607844, 0.6941176470588235, 0.8980392156862745),
            (0.4627450980392157, 0.5686274509803921, 0.1843137254901961),
            (0.0, 0.19215686274509805, 0.03529411764705882),
            (0.0, 0.3764705882352941, 0.803921568627451),
            (0.8235294117647058, 0.0, 0.5882352941176471),
            (0.5372549019607843, 0.3333333333333333, 0.38823529411764707),
            (0.1607843137254902, 0.12549019607843137, 0.11372549019607843),
            (0.3568627450980392, 0.19607843137254902, 0.07450980392156863),
            (0.6549019607843137, 0.43529411764705883, 0.25882352941176473),
            (0.5372549019607843, 0.2549019607843137, 0.1803921568627451),
            (0.10196078431372549, 0.22745098039215686, 0.16470588235294117),
            (0.28627450980392155, 0.29411764705882354, 0.35294117647058826),
            (0.6588235294117647, 0.5490196078431373, 0.5215686274509804),
            (0.9568627450980393, 0.6705882352941176, 0.6666666666666666),
            (0.6392156862745098, 0.9529411764705882, 0.6705882352941176),
            (0.0, 0.7764705882352941, 0.7843137254901961),
            (0.9176470588235294, 0.5450980392156862, 0.4),
            (0.5843137254901961, 0.5411764705882353, 0.6235294117647059),
            (0.7411764705882353, 0.788235294117647, 0.8235294117647058),
            (0.6235294117647059, 0.6274509803921569, 0.39215686274509803),
            (0.7450980392156863, 0.2784313725490196, 0.0),
            (0.396078431372549, 0.5058823529411764, 0.5333333333333333),
            (0.5137254901960784, 0.6431372549019608, 0.5215686274509804),
            (0.27058823529411763, 0.23529411764705882, 0.13725490196078433),
            (0.2784313725490196, 0.403921568627451, 0.36470588235294116),
            (0.22745098039215686, 0.24705882352941178, 0.0),
            (0.023529411764705882, 0.07058823529411765, 0.011764705882352941),
            (0.8745098039215686, 0.984313725490196, 0.44313725490196076),
            (0.5254901960784314, 0.5568627450980392, 0.49411764705882355),
            (0.596078431372549, 0.8156862745098039, 0.34509803921568627),
            (0.4235294117647059, 0.5607843137254902, 0.49019607843137253),
            (0.8431372549019608, 0.7490196078431373, 0.7607843137254902),
            (0.23529411764705882, 0.24313725490196078, 0.43137254901960786),
            (0.8470588235294118, 0.23921568627450981, 0.4),
            (0.1843137254901961, 0.36470588235294116, 0.6078431372549019),
            (0.4235294117647059, 0.3686274509803922, 0.27450980392156865),
            (0.8235294117647058, 0.3568627450980392, 0.5333333333333333),
            (0.3568627450980392, 0.396078431372549, 0.4235294117647059),
            (0.0, 0.7098039215686275, 0.4980392156862745),
            (0.32941176470588235, 0.3607843137254902, 0.27450980392156865),
            (0.5254901960784314, 0.3764705882352941, 0.592156862745098),
            (0.21176470588235294, 0.36470588235294116, 0.1450980392156863),
            (0.1450980392156863, 0.1843137254901961, 0.6),
            (0.0, 0.8, 1.0),
            (0.403921568627451, 0.3058823529411765, 0.3764705882352941),
            (0.9882352941176471, 0.0, 0.611764705882353),
            (0.5725490196078431, 0.5372549019607843, 0.4196078431372549))]

names = ('cartercarter_6', 'greenarmytage_26', 'tatarize_269')
colorlists = [base_objects.rgba2colorlist(color_set) for color_set in colors]
cset_dict = dict(zip(names, colorlists))
custom_colorsets = collections.namedtuple('colorsets', cset_dict)(**cset_dict)


